import com.ibm.websphere.rainmaker.logging.Output;
import java.util.logging.Logger;
import com.ibm.ws.xs.app.DomainConnectionUpdater;


def createDomain(domain, owner = false) {
    if (domain.name == '') {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.domain.name.required', null);
    }
    if (domain.name == "\\") {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.singleslash', null);
    }
	
	
    def filters = [name:domain.name];
    def domains = getDomains(filters);
    if (domains.size() > 0) {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.domain.alreadyexists', null, [domain.name]);
    }
	
    if (getDomains([:], false).size() == 0) {
        storage.current.domain = domain.name
    }
    DomainConnectionUpdater.setConnectionInfo(DomainConnectionUpdater.CURRENT_DOMAIN);
        
    def resource = createResource('domains', domain, owner);
    if (resource.errorMessage != null && resource.errorMessage != '') {
        if (resource.errorMessage =~ /length/) {
            def lengthValue = resource.errorMessage.split("length ");
            return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.property.length', null, [lengthValue[1]]);
        }
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.domain.create', null, [resource.errorMessage]);
    } else {
        return resource;
    }
}

def deleteDomain(id) {
    def dom = getDomain(id)

    if (dom != null) {
        if (dom.name.equals(storage.current.domain[])) {
            storage.current.domain = ""
        }
    }
    def rv = deleteResource('domains', id)
    def refs = getResources('domainsservers', [domainid: id], false)

    refs.each {
        deleteResource('domainsservers', it.id)
    }

    refs = getResources('domainskeystores', [domainid: id], false)

    refs.each {
        deleteResource('domainskeystores', it.id)
    }

    return rv
}

def getDomain(id) {
    //invokeMethod("groovy/rainmaker/keystores/Keystores.groovy", "refreshKeystores", null);
    def rv = getResource('domains', [id: id], ['id', 'name', 'jmxuser', 'jmxpass',  'customcredgenerator', 'credgeneratorproperties', 'serverprops'])
    
    return rv
}

def getDomains(filters, owner = false) {
    def _filters = filters.clone()
    def rv = getResources( 'domains', _filters, false, ['id', 'name', 'jmxuser', 'jmxpass', 'customcredgenerator', 'credgeneratorproperties', 'serverprops'])

    return rv
}

def updateDomain(domain) {
    // Map verify password to normal password update as expected by updateResource.
    if (domain.verification != null) {
       domain.put("keystorepassword", domain.value);
       domain.remove("value");
       domain.remove("verification");
       domain.remove("widget");
    }
    def rv = updateResource('domains', domain, ['id', 'name', 'jmxuser', 'jmxpass', 'customcredgenerator', 'credgeneratorproperties', 'serverprops'])
    DomainConnectionUpdater.setConnectionInfo(DomainConnectionUpdater.CURRENT_DOMAIN);
    
    return rv
}

def sanitizeCatserverRec(rv) {
     // Create a version that's safe to write out to the logs.
     def sanitized = rv.clone().collect() { it.clone() }
     sanitized.each { srv ->
         if (srv.security.jmxPass != null) {
             srv.security = srv.security.clone()
             srv.security.jmxPass = "******"
         }
         if (srv.jmxPort != null)
        	 srv.jmxPort = ""
     }
     return sanitized
 }

def convertDomainToCatserverRec(filters) {
    def hits = getDomains(filters, false)

    if (hits.size() == 0) {
        return null
    }

    def domain  = hits[0]
    def servers = getDomainsservers([domainid: domain.id], false)
    def rv      = []

    servers.each { srv ->
        def server = getResource("servers", [id: srv.serverid], ["alias", "name", "listenerport", "jmxport"])

        if (server != null) {
            def x = [ 
                serverName: server.alias?.trim(), 
                hostName: server.name?.trim(), 
                orbPort: server.listenerport.toString(),
                //jmxPort: server.jmxport.toLong(),
                security: [ 
                    customCredGenerator: domain.customcredgenerator?.trim(),
                    credGeneratorProperties: domain.credgeneratorproperties?.trim(),
                    serverProps: domain.serverprops?.trim() ] ]

            rv << x
        } else {
          Output.error("Bad server id ${srv.serverid}, ignoring")
        }
     }

     
	 logger.FINE { "Conversion: ${sanitizeCatserverRec(rv)}" };
     return rv
}
