/* TODO: This should move to a xs.app.setup, so it is easier for us to not ship this. */

import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.users.Groups;
import com.ibm.websphere.users.Users;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.FileOutputStream;

def taskLogger;
def lang;

def createMonitorAdmin()
{
    def user = [
        username: 'admin', 
        name: MessageUtils.message(lang, 'XSW0111', []),
        email: ''
    ];

    try
    {
        Users.createUser(user, false, config.rainmaker.admin.defaultPassword[]);
        Users.addUserRole(user.id.toInteger(), Permissions.SUPER_USER);
        Users.addUserRole(user.id.toInteger(), Permissions.APPLIANCE_ADMIN);
        Users.addUserRole(user.id.toInteger(), Permissions.APPLIANCE_MONITOR);
        Users.addUserRole(user.id.toInteger(), Permissions.GRID_CREATE);
        return true;
    }
    catch (Throwable t)
    {
        broadcastError(MessageUtils.message(lang, 'XSW0167', [t.getMessage()]));
		log(CleanStackTrace.cleanStackTrace(t));
    }
    return false;
}


def mkUserTables(newestVersion) {
    def gendir = new File(config.app.root[]);

	log("Creating new XS database using schema version: "+newestVersion);
    create(gendir, ".", "derby", newestVersion);

    log("Creating acl tables");
    create(gendir, config.dependencies.'ibm:rainmaker.acl'[], "derby", "1.0.0.0");
    createResource("Version", [id: 1, number: newestVersion], null, null);
}

def popUserTables() {
    config.xsa.retrieveFromBackupZone = false;
    config.rainmaker.alwaysJoinThreads = true;
    
    log('Creating all-inclusive public group...');        
    Groups.createEveryoneGroup();

    log('The public group was created successfully.');
    
    log('Creating admin account...');        
    if (createMonitorAdmin()) {
        log('The admin account was created successfully.');
        return true;
    }
	return false;
}        


def mkSecretKey() {
    secretKeyConfigFile = new File(config.dependencies.'ibm:xs.app'[] + "/config/key.config");
    def secretKey = zero.core.security.encryption.SecretKeyGenerator.generateSecretKey()
    secretKeyConfigFile.write ("/config/security/secretKey=\"${secretKey}\"")
}

def migrateTables(newestVersion, allVersions) {
	//No way to role back.  Need to be able to look up version and role back.
	def version = getResource("Version", [id: 1], null);
	if (version == null) {
		broadcastInfo(MessageUtils.message(lang, 'XSW0160', []));
		version = [number: "1.0.0.0"];
	}

    if (isVersionLessThan(version.number, newestVersion)) {
    	log("XS database schema is not at latest version.");
    	migrateTablesHelper(version, newestVersion, allVersions);
    } else {
    	log("XS database schema version: "+version.number);
    }
}

def migrateTablesHelper(version, newestVersion, allVersions) {
    def gendir = new File(config.app.root[]);
	def startMigrating = false;
	try {
		allVersions.each { migrationVersion ->
			if (startMigrating) {
				log("Migrating XS database schema to: "+migrationVersion);
		    	migrate(gendir, ".", "derby", migrationVersion);
		    	migrateData(migrationVersion);
		    	log("Migration completed successfully.");
			}
			
			if (version.number == migrationVersion)
				startMigrating = true;
		}

		if (version.number == "1.0.0.0") {
	    	createResource("Version", [id: 1, number: newestVersion], null, null);
	    	log("Version table added successfully");
		} else {
			updateResource("Version", [id: 1, number: newestVersion], null);
			log("Version table updated to "+newestVersion);
		}
	} catch (e) {
		broadcastError(MessageUtils.message(lang, 'XSW0159', [taskLoggerPath]));
		log(CleanStackTrace.cleanStackTrace(e));
	}
}

def migrateData(version) {
	if (version == "1.0.0.2") {
		migrateData1002();
	}
}

def migrateData1002() {
	def filters = [name:null, order:"name", count:null, page:null] ;
	def servers = getResources( 'servers', filters, false,  ["id", "alias", "name", "listenerport", "jmxport"])
	servers.each { server ->
		if (server.alias == null) {
			server.alias = server.name+":"+server.listenerport;
			updateResource('servers', server,  ["id", "alias", "name", "listenerport", "jmxport"]);
			log("Catalog server "+server.name+" migrated to 1.0.0.2 DB schema.");
		}
	}
}

def isVersionLessThan(version, currentVersion) {
	def normalizedVersion = version.replaceAll("\\.", "");
	def normalizedCurrentVersion = currentVersion.replaceAll("\\.", "");	
	if (normalizedVersion < normalizedCurrentVersion) {
		return true;
	} else {
		return false;
	}
}

def broadcastInfo(message) {
	Output.info(message);
	log(message);
}

def broadcastError(message) {
	Output.error(message);
	log(message);
}

def log(message) {
	taskLogger.append(getNow()+" :: "+message+"\n");
}

def getNow() {
	SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
	return formatter.format(Calendar.getInstance().getTime());
}

def onCliTask() {
	taskLoggerPath = config.dependencies.'ibm:xs.app'[] + "/logs/startConsoleServer.log"
	lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
	taskLogger = new File(taskLoggerPath);
	def allVersions = ["1.0.0.0", "1.0.0.1", "1.0.0.2"];
	def newestVersion = "1.0.0.2";

	if (taskLoggerPath.indexOf("\\") != -1) {
		taskLoggerPath = taskLoggerPath.replace("\\", "\\\\").replace("/", "\\\\");
	}

    try {
        def f = new File(config.app.root[] + "/databases")
        if (!f.exists()) {
            mkSecretKey()
            mkUserTables(newestVersion)
            if (!popUserTables()) {
                config.exitCode = 1
                return;
            }
        } else {
        	migrateTables(newestVersion, allVersions)
        }
    } catch (Throwable t) {
    	broadcastError(MessageUtils.message(lang, 'XSW0159', [taskLoggerPath]));
		log(CleanStackTrace.cleanStackTrace(t));
        config.exitCode = 1
        return;
    }
	renamePortConfigFile();
}

//Defect 12800
def renamePortConfigFile(){
	def oldFile = new File(config.dependencies.'ibm:xs.app'[] + "/config/temp-port-config.config");
	if(oldFile.exists()) {
		//Rename file to the correct file
		def newFile = new File(config.dependencies.'ibm:xs.app'[] + "/config/port-config.config")
		if(newFile.exists()) newFile.delete();
		oldFile.renameTo(newFile);
	}
}
