/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;
import com.ibm.websphere.rainmaker.appliance.AggregateInterfaces;
import com.ibm.bedrock.config.ReturnCode;
import com.ibm.websphere.rest.RESTUtils;

def onList()
{ 
    if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    try
    {
        def interfaceNames = AggregateInterfaces.getAggregateInterfaces();
    
        def results = [];
        
        interfaceNames.each() { current ->
        		results.add(AggregateInterfaces.getAggregateInterface(current));
            }
        
        request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
        request.json.output = results;
        render();
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08000', t);
    }
}

def onRetrieve()
{       
    def results = null;
    
    if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    try
    {
        String id = request.params.aggregateInterfacesId[];
		def aggregateInterface = AggregateInterfaces.getAggregateInterface(id); 
		if(aggregateInterface!= null) { // Use ViewEngine JSON rendering 
			request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
			request.json.output = aggregateInterface;
			render();
		} else { // Error handling 
			request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "aggregateInterface $id not found."; 
			request.view = 'error'; 
			render(); 
		} 
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08049', t);
    }
}

def onUpdate()
{       
    def json = null;
    
    if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    try
    {
        json = Json.decode(request.input[]);
        if (!com.ibm.websphere.rest.RESTUtils.areValidParameters(json)){
    		return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
    	}
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        logger.FINEST { "Executing method update." };
        ReturnCode retCode = AggregateInterfaces.updateAggregateInterface(json);
        if ( retCode != ReturnCode.kwOK ){
        	request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "Could not update aggregate interface $json.name ."; 
			request.view = 'error'; 
			render(); 
        }
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08004', t);
    }
}

def onCreate() 
{ // Convert entity to JSON object 
	def json = null;
	
	
	if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    try
    {
        json = Json.decode(request.input[]);
        if (!com.ibm.websphere.rest.RESTUtils.areValidParameters(json)){
    		return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
    	}
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        logger.INFO { "Executing create aggregateInterface." };
        ReturnCode retCode = AggregateInterfaces.createAggregateInterface((Map)json);
        logger.INFO {"Return Code = "+ retCode.name()};
        if ( retCode != ReturnCode.kwOK ){
        	request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "Could not create aggregate interface $json.name ."; 
			request.view = 'error'; 
			render(); 
        } else {
        	locationUri = getRequestedUri(false) + '/' + json.name; 
			request.headers.out.Location = locationUri ;
			request.status = HttpURLConnection.HTTP_NO_CONTENT;
		} 
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08004', t);
    }
} 

def onDelete()
{       
    def results = null;
    
	if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    try
    {
    	logger.FINEST { "Executing method update aggregateInterface." };
        String id = request.params.aggregateInterfacesId[];
		ReturnCode retCode = AggregateInterfaces.deleteAggregateInterface(id);
		logger.INFO {"Return Code = "+ retCode.name()};
		if ( retCode != ReturnCode.kwOK ){
        	request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "Could not delete aggregate interface $id ."; 
			request.view = 'error'; 
			render(); 
        }
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08049', t);
    }
}

def onPostMember()
{   
    def json = null;
    
	if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    try
    {
        json = Json.decode(request.input[]);
        if (!com.ibm.websphere.rest.RESTUtils.areValidParameters(json)){
    		return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
    	}
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        logger.FINEST { "Executing method update aggregateInterface." };
        ReturnCode retCode = AggregateInterfaces.updateAggregateInterface((Map)json);
        if ( retCode != ReturnCode.kwOK ){
        	request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "Could not update aggregate interface $json.name ."; 
			request.view = 'error'; 
			render(); 
        } else {
        	locationUri = getRequestedUri(false) + '/' + request.params.name; 
			request.headers.out.Location = locationUri ;
			request.status = HttpURLConnection.HTTP_NO_CONTENT;
		} 
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08004', t);
    }
}
