/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// ListView represents the lefthand side of each page, where a list of resources is displayed. 
// The list may be filtered or sorted using the FilterBox widget that is embedded inside it.
// This widget also coordinates changes to the righthande side of the page (DetailsView), which 
// shows low-level information about resources selected from the list sidebar.
//

dojo.provide('javascript.widgets.ListView');
dojo.require('javascript.widgets.FormDialog');
dojo.require("javascript.widgets.FilterBox");
dojo.require('javascript.widgets.BaseWidget');
dojo.require('javascript.widgets.ModalDialog');

dojo.declare('javascript.widgets.ListView', javascript.widgets.BaseWidget, {   

    _createAddIcon: function(parentDiv) {
        var addDiv = document.createElement('div');
        dojo.addClass(addDiv, 'dashboard-list-add-resource');
        parentDiv.appendChild(addDiv);
        
        var link = document.createElement('a');
		link.setAttribute('href', 'javascript:void(0);');
        
		link.onclick = dojo.hitch(this, this.createResource);
        
        addDiv.appendChild(link);
        
        var icon = document.createElement('img');
        icon.setAttribute('id', 'addIcon');
		icon.setAttribute('src', '/images/add-icon.gif');
		icon.setAttribute('alt', messages['RM11040']);
		icon.setAttribute('title', messages['RM11040']);
		dojo.addClass(icon, 'icon-image');
        link.appendChild(icon);
    },
    
    _createDeleteAllIcon: function(parentDiv, actionSize, actionItem) {
        var addDiv = document.createElement('div');
        var rightPadding = 12+((actionSize-1)*24)+'px';
		addDiv.setAttribute('style', 'position: absolute; top: 6px; right: '+rightPadding);
        parentDiv.appendChild(addDiv);
        
        var link = document.createElement('a');
		link.setAttribute('href', 'javascript:void(0);');
        
		link.onclick = dojo.hitch(this, this.deleteAllResource, actionItem);
        
        addDiv.appendChild(link);
        
        var message = messages['RM11123'];
        if (actionItem.label != undefined) {
        	message = messages[actionItem.label];
        }
        
        var icon = document.createElement('img');
        icon.setAttribute('id', 'deleteAllIcon');
		icon.setAttribute('src', '/images/ibm/Eraser-Clear_16.png');
		icon.setAttribute('alt', message);
		icon.setAttribute('title', message);
		dojo.addClass(icon, 'icon-image');
        link.appendChild(icon);
    },

    _createSuccessful: function(createType, response) {
        if (createType.create.onSuccess)
            createType.create.onSuccess(response);
        
        this.updateResources();

		var id = null;
		if ( response.id ) {
			id = response.id;
		}
		else if ( response.taskId ) {
			id = response.taskId;
		}
		else {
        	var location = response.getHeader('Location');
        	id = location.split('/').pop();
        }
        setTimeout(dojo.hitch(this, this.selectResource, id), 1000);
    },
    
    //
    // createResource() pops up a dialog that allows users to create a new resource, one with 
    // the same type as objects currently shown in the list.
    //
    createResource: function() {
        if (document.dialog)
            return;

        try {
	        var type = this._getResourceType();
    	    var callback = dojo.hitch(this, this._createSuccessful, type);
        	var dialog = new javascript.widgets.FormDialog('create-dialog', type.create.text, type.uri, type.create.fields, callback, 'POST', type.create.jsonBuilder);
	        dialog.show();
        }
        catch ( e ) {
           console.log(e);
        }    
    },
    
    deleteAllResource: function(id) {
        if (document.dialog) {
            return;
        }
            
        var submitHandler = dojo.hitch(this, this.deleteAllRequest);
        
        var message = messages['RM11124'];
        if (id.label != undefined) {
        	message = messages[id.dialogLabel];
        }
        
        this.dialog = new javascript.widgets.ModalDialog('modal-dialog', message, submitHandler);
        this.dialog.show();
    },
    
    deleteAllRequest: function() {
    	console.log("mjs1", [this._getResourceType().uri+'/all']);
    	console.log("mjs2", [this.dialog]);
        var xhrArgs = {
            url: this._getResourceType().uri+'/all',
            content: {},
            sync: true,
            handleAs: "json",
            headers: {"Content-Type" : "text/json"},
            load: dojo.hitch(this, this.deleteAllSuccess)
        };

        var result = dojo.xhrDelete(xhrArgs);
        console.log("mjsEntry", [result]);
    },
    
    deleteAllSuccess: function() {
    	console.log("deleteAllSuccess", this);
    	console.log("deleteAllSuccess2", this.dialog);
        this.dialog.hide();
        if (this.details)
            this.details.showDefault(this.uri);
        this.parent.list.updateResources();    	
    },
    
    _getResourceType: function() {
        for (var i = 0; i < this.resourceTypes.length; ++i)
            if (this.resourceTypes[i].uri == this.uri)
                return this.resourceTypes[i];
        
        return null;
    },
    
    _highlightResource: function(id, highlight) {        
        var element = dojo.byId('dashboard-list-view-resource-' + id);
        
        if (element == null)
            return;
        
        var func = highlight ? Bedrock.addClassName : Bedrock.removeClassName;
        func(element.id, 'dashboard-list-view-resource-selected');
    },
    
    
    constructor: function(parent, details, resourceTypes, options) {
        // JMB, can only extend one level with the code like this...
        this.initialize(parent, { 
            id: 'list-view', 
            'class': 'dashboard-list-view' 
        });
        
        console.log("ListView constructor", [details, resourceTypes, options]);
        var defaults = {
            selectableResources: true, 
            create: resourceTypes[0].create != undefined,
            actions: resourceTypes[0].actions != undefined,
            uri: resourceTypes[0].uri,
            sort: true,
			menuOptions : [
	            {
    	            field: 'name',
        	        nls: messages['RM11038']
            	},
            	{
                	field: 'currentstatus',
                	nls: messages['RM11039']
            	}
        	]
        }	
        
		Bedrock.addProperties(defaults, options || {});
        //Object.extend(defaults, options || {});
        
        this.details = details;
        if ( details.panel && details.panel.setListUpdate ) {
	        details.panel.setListUpdate(dojo.hitch(this, this.updateResources));
        }
        
        this.resourceTypes = resourceTypes;
        this.uri = defaults.uri;
        
        var headerID = this.element.id + '-header';
        
        var headerDiv = document.createElement('div');
		headerDiv.setAttribute('id', headerID);
		dojo.addClass(headerDiv, 'dashboard-list-view-header');
        this.element.appendChild(headerDiv);
        
        if (defaults.summary) {
            headerDiv.innerHTML = "<span aria-level='1' role='heading' class='dashboard-list-view-header-message'>" + defaults.summary + "</span>";
        }
        
        var actionSize = 0;
        if (defaults.create ) {
        	if ( resourceTypes[0].create.enabled == undefined || resourceTypes[0].create.enabled ) {
	            this._createAddIcon(headerDiv);
	            actionSize = 1;
        	}
        }

        if (defaults.actions) {
        	for (var i=0; i<resourceTypes[0].actions.length; i++) {
        		if (resourceTypes[0].actions[i].enabled == undefined || resourceTypes[0].actions[i].enabled ) {
        			if (resourceTypes[0].actions[i].name == "deleteAll") {
        				this._createDeleteAllIcon(headerDiv, actionSize+i+1, resourceTypes[0].actions[i]);
        			}
        		}
        	}
        }
        
        //
        // HACK: use alternate scrollable class for pattern editor 
        //
        var scrollableClass = resourceTypes.length > 1 ? 'alt-scrollable' : 'scrollable';
        var resourcesID = this.element.id + '-resources';
        var resourcesDiv = document.createElement('div');
		resourcesDiv.setAttribute('id', resourcesID);
		dojo.addClass(resourcesDiv, 'dashboard-list-view-resources ' + scrollableClass);
                
        if (window.location.href.match(/editor/))
            resourcesDiv.style.overflowY = 'visible';
        
        this.element.appendChild(resourcesDiv);
        
        var label = document.createElement('label');
        label.setAttribute('style', 'display: none;');
        label.innerHTML = messages['RM11000'];
        this.element.appendChild(label);

        this.filter = new javascript.widgets.FilterBox(this, resourcesDiv, resourceTypes, defaults);

        label.setAttribute('for', this.filter.element.id+"-text");

        var divider = document.createElement('span');
        dojo.addClass(divider, 'dashboard-divider');

        this.element.appendChild(divider);
        
        parent.list = this;
    },

       
    initialize: function(parent, details) {
    	this.inherited(arguments );
    },
    
    selectResource: function(id) {
        console.log("list-view, select resource");
        if (this.currentSelectedID != id)
        {
            this._highlightResource(this.currentSelectedID, false);
            this._highlightResource(id, true);
            this.currentSelectedID = id;
            
            window.location.hash = id;
        }
        
        if (this.details)
            this.details.selectResource(this.uri, this.currentSelectedID);
    },
    
    selectResourceType: function(uri) {
        this.uri = uri;
        this.currentSelectedID = null;
        this.filter.toggle(this.uri);

        if (this.details)
            this.details.showDefault(this.uri);
    },

    setAjaxParameter: function(key, value) {
        this.filter.setAjaxParameter(key, value);
    },
    
    showPage: function(page) {
        this.filter.showPage(page);
    },
    
    //
    // toggleResources() changes which REST API is used to collect data and render the list.
    //
    toggleResources: function() {
        this.selectResourceType(dojo.byId('toggle-menu').value);
    },
    
    //
    // updateResources() forces a refresh of the list content based on the current filter text.
    //
    updateResources: function() {
        this.filter._getUpdatedChoices();
    }
    
});
