/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.layout.SingleGridViewPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Select");
dojo.require("dijit.layout.ContentPane");
dojo.require("javascript.widgets.GenericDialog");
dojo.require("xsa.monitor.widgets.resizing.ResizingChartWithTableData");
dojo.require("xsa.datamgmt.widgets.QueryGrid2");
dojo.require("dojo.window");

dojo.requireLocalization("xsa.datamgmt.widgets", "datamgmt_messages");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.layout.SingleGridViewPane", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/layout/templates/SingleGridViewPaneTemplate.html"),
	widgetsInTemplate: true,
	
	subscriptions:null,
	gridName:null,
	maps:null,
	datamgmt_messages: null,
	xsa_messages: null,
	chartValue: "",
	chart: null, //small chart
	largeChart: null, //larger chart shown in dialog
	resizableChart: null,
	fill: false,
	showQueryConsole: false,
	url: null,
	aggregationScope:null,
	metrics:null,
	yAxisLabel:"",
	xAxisLabel:"",
	
	chartType:"line",
	filterSessionMaps: true,
	filterSimpleMaps: true,
	isSession: false,
	gridInUse: true,  //Assume true as it will be set later
		
	constructor: function(args){	
		this.subscriptions = [];
		this.gridName = args.gridName;
		this.chartName = args.chartName;
		if(args.chartValue) this.chartValue = args.chartValue;
		this.url = args.url;
		this.aggregationScope = args.aggregationScope;
		this.metrics = args.metrics;
		
		this.chartArray = args.chartArray;
		if(args.yaxis) this.yaxis = args.yaxis;
		if(args.yAxisLabel) this.yAxisLabel = args.yAxisLabel;
		if(args.xAxisLabel) this.xAxisLabel = args.xAxisLabel;
		if(args.offset) this.offset = args.offset;
		if(args.maps) this.maps = args.maps;
		
		this.datamgmt_messages = dojo.i18n.getLocalization("xsa.datamgmt.widgets", "xsa_messages");
		this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
		
		this.search = this.datamgmt_messages.searchIconHover;
		this.openLargerChart = this.xsa_messages.openLargerChart;
		this.backToOverview = this.xsa_messages.backToOverview;
		
		dojo.forEach(this.maps, function(map){
			if(map == "objectgridSessionMetadata") {
				this.isSession = true;
			}
		}, this);
	},	
	
	postCreate: function(){ //The parent container SingleGridViewLayour has not yet been initialized at this time
		dojo.place(this.chart.domNode, this.chartDiv, "first");
		
		//setup large chart content pane
		this.largeChartContentPane = dojo.create("div", {'class':"chartContentPane"});
		dojo.place(this.largeChartContentPane, dojo.byId("layoutDiv"), "after");
		var button = dojo.create("button", {'class':"btn btn-primary right", innerHTML:this.backToOverview, role:"button", 'aria-label':this.backToOverview});
		this.buttonConnection2 = dojo.connect(button, "onclick", this, "goBack");
		dojo.place(button, this.largeChartContentPane, "first");
		this.largeChartCP = new dijit.layout.ContentPane({}, this.largeChartContentPane);
		dojo.attr(this.largeChartCP.domNode, "role", "region");
        dojo.attr(this.largeChartCP.domNode, "aria-label", this.chartName + " " + this.xsa_messages.chartWithHourView);

		dojo.place(dojo.create("div", {innerHTML:this.chartName,
						'class': "singleGridLabelWhite"
		}), this.largeChartContentPane, "last");
	
		if(!this.showQueryConsole) {
			dojo.style(this.searchButton, "display", "none");
		}
		
		var l = "en";
		try{
			if(djConfig && djConfig.locale) l = djConfig.locale;
		} catch (e) {
			console.log("Tooltip date is defaulting to english locale");
		}
		//Update the current value
		if(this.chart.chartType=="line" && this.chart && this.chart.chartArray[0] && this.chart.chartArray[0].data[0]){
			var data = this.chart.chartArray[0].data[this.chart.chartArray[0].data.length-1][1].toFixed(2);
			data = dojo.number.format(this.chart.chartArray[0].data[this.chart.chartArray[0].data.length-1][1], { locale: l});
			
			this.currentValueDiv.innerHTML = data+"&nbsp;"+this.chartValueLabel.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
		}
		this.subscriptions[this.subscriptions.length] =  dojo.subscribe(this.chart.id+"_chartArrayUpdated", dojo.hitch(this, function(message){
			if(this.chart.chartType=="line"){
				var data = message.chartArray[message.chartArray.length-1][1].toFixed(2);
				data = dojo.number.format(data, { locale: l});
				
				this.currentValueDiv.innerHTML = data + "&nbsp;"+this.chartValueLabel.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
				dojo.attr(this.domNode.parentNode, "aria-label", this.chartName + " " + this.xsa_messages.currentValueLabel + " " + this.currentValueDiv.innerHTML.split("&nbsp;").join(" "));
			} else if(this.chart.chartType=="pie"){
				var pieLabel = dojo.query(".pieChartLabel");
				if(pieLabel[0]) {
					dojo.attr(this.domNode.parentNode, "aria-label", this.chartName + " " + this.xsa_messages.currentValueLabel + " " + pieLabel[0].innerHTML.split("&nbsp;").join(" ").split("<br>").join(" "));
				}
				this.postGridNotUsedInfoMessage(message.chartArray[0][1]);
			}
		}));
		
		//create the tooltips
		this.openLargeChartTooltip = new dijit.Tooltip({
			connectId: this.openLargeChartButton,
			label: this.openLargerChart
		});
		if(this.showQueryConsole){
			this.openQueryConsoleTooltip = new dijit.Tooltip({
				connectId: this.searchButton,
				label: this.search
			});
		}
		
	},
	
	displayed: function(){ //used by the parent container to tell this widget that it has been visually displayed
		//We know the parent container is initialized now, so lets try to display info message if the grid is not in use
		if(this.chart.chartType=="pie" && this.chart && this.chart.chartArray[0] && this.chart.chartArray[0].data[0]){
			this.postGridNotUsedInfoMessage(this.chart.chartArray[0].data[0][1]);
		}
		if(this.chart.chartType=="line"){
			dojo.attr(this.domNode.parentNode, "aria-label", this.chartName + " " + this.xsa_messages.currentValueLabel + " " + this.currentValueDiv.innerHTML.split("&nbsp;").join(" "));
		} else if(this.chart.chartType=="pie") {
			var pieLabel = dojo.query(".pieChartLabel");
			if(pieLabel[0]) {
				dojo.attr(this.domNode.parentNode, "aria-label", this.chartName + " " + this.xsa_messages.currentValueLabel + " " + pieLabel[0].innerHTML.split("&nbsp;").join(" ").split("<br>").join(" "));                                                          
			}
		}
	},
	
	postGridNotUsedInfoMessage: function(currentValue){
		//post message if grid not in use OR remove message if it is in use
		if(currentValue==0){
			this.gridInUse = false;
			dojo.publish("updateInfoMessage", [{
				message: this.isSession ? this.datamgmt_messages.sessionGridIsNotInUseInfo : this.datamgmt_messages.gridIsNotInUseInfo
			}]);
			if(this.queryConsoleCP){
				var queryGrid = dijit.byId("queryConsoleWidget");		
				if(dojo.style(queryGrid._infoMessage, "display")=="none"){
					//display grid is not in use message
					queryGrid._infoMessage.innerHTML = queryGrid.xsa_messages.gridIsNotInUseInfo;
					dojo.style(queryGrid._infoMessage, "display", "");
					dojo.style(queryGrid._errorMessage, "display", "none");
					dojo.style(queryGrid._resultsTable, "display", "none");
					dojo.style(queryGrid.tableMessage, "display", "none");
					dojo.style(queryGrid.headerTable, "display", "none");
				} else {
					//do nothing.  Panel is already displayed correctly
				}
			}
		} else {
			this.gridInUse = true;
			dojo.publish("updateInfoMessage", [{
				message:""
			}]);
			
			if(this.queryConsoleCP){
				var queryGrid = dijit.byId("queryConsoleWidget");
				if(dojo.style(queryGrid._infoMessage, "display")!="none" && dojo.style(queryGrid._resultsTable, "display")=="none"){
					//info message is displayed and there is no previous query
					dojo.style(queryGrid._infoMessage, "display", "none");
					dojo.style(queryGrid.tableMessage, "display", "");
					dojo.style(queryGrid.headerTable, "display", "");
				} else {
					//do nothing.  Panel is already displayed correctly
				}
			}
		}
	},
	
	mouseEnter: function(){
		document.body.style.cursor = "pointer";	
	}, 
					
	mouseLeave: function(){
		document.body.style.cursor = "";
	}, 
										
	lcKeyPress: function(e){
		if(this.isEnter(e.keyCode)) {
			this.openLargeChart();
		}
	},
	
	qcKeyPress: function(e){
		if(this.isEnter(e.keyCode)) {
			this.openQueryConsole();
		}
	},
	
	isEnter: function(keyCode) {
		return keyCode == dojo.keys.ENTER ? true : false;
	},
	
	createQueryConsole: function(){
		this.queryConsoleContentPane = dojo.create("div", {'class':"chartContentPane"});
		dojo.place(this.queryConsoleContentPane, dojo.byId("layoutDiv"), "after");
		var button = dojo.create("button", {'class':"btn btn-primary right", innerHTML:this.backToOverview, role:"button", 'aria-label':this.backToOverview});
		this.buttonConnection = dojo.connect(button, "onclick", this, "goBack");
		dojo.place(button, this.queryConsoleContentPane, "first");
		this.queryConsoleCP = new dijit.layout.ContentPane({}, this.queryConsoleContentPane);
		dojo.attr(this.queryConsoleCP.domNode, "role", "region");
        dojo.attr(this.queryConsoleCP.domNode, "aria-label", this.datamgmt_messages.searchForKey);
		
		var mapName = this.isSession ? "objectgridSessionMetadata" : "";
		
		if(!this.isSession || !this.filterSessionMaps) { //Not a Session Grid or filtering is turned off
	//		var div  = dojo.create("div", {'class':'styled-select dijitInline'}); //uncomment this our when the cloud css styled-select is working better
			var div  = dojo.create("div", {'class':'dijitInline'});
			this.mapSelection = dojo.create("select", {name: "mapName", id:"mapName"});
			dojo.place(this.mapSelection, div, "first");
			
			connection = dojo.connect(this.mapSelection, "onchange", dojo.hitch(this, function(){
				if(this.queryConsole) {
					this.queryConsole.mapName = this.mapSelection.value;
				}
			}));
		
			//build options for the select box
			dojo.forEach(this.maps, function(map, index){
				var opt = dojo.create("option", {value: map, innerHTML: map});
				if(index==0) dojo.attr(opt, "SELECTED", "true");
				dojo.place(opt, this.mapSelection, "first");
			}, this);
			mapName = this.mapSelection.value;
		}
		
		this.queryConsole = new xsa.datamgmt.widgets.QueryGrid2({id:"queryConsoleWidget", gridName: this.gridName, 
									mapName: mapName,
									gridInUse: this.gridInUse
									});
		dojo.place(this.queryConsole.domNode, this.queryConsoleContentPane, "last");
		dojo.attr(this.queryConsole.searchContainerTableCell, "align", "right");
	
		if(!this.isSession || !this.filterSessionMaps) {
			//Place the map selection and it's label into the dom
			var mapTD = dojo.create("td", {align:"left"});
			dojo.place(mapTD, this.queryConsole.searchContainerTableCell, "before");
		
			dojo.place(dojo.create("label", {'for':"mapName", innerHTML:this.xsa_messages.mapLabel+":&nbsp;&nbsp;", style:{color:'white',marginTop:0}, 'class':"dijitInline"}), mapTD, "first");
			dojo.place(div, mapTD, "last");
		}
	},
	
	resize: function(height, width){
		if(this.chart.chartType=="pie") {
			this.chart.resize(height-60, width-40);
		} else {
			this.chart.resize(height-40, width-40);
		}
		this.resizeLargeChart();	
	},
	
	resizeLargeChart: function(){
		var windowSize = dojo.window.getBox();
		if(this.resizableChart) {
			this.resizableChart.resize(windowSize.h*0.85, windowSize.w*.97);
		}
	}, 
	
	createLargeChart: function() {
		//Create the chart
		dojo.style(this.largeChartContentPane, "display", "block");
		if(this.resizableChart) this.resizableChart.destroyRecursive();
		
		var offset =  this.chart.chartArray[0].unitOffset ? this.chart.chartArray[0].unitOffset : "undefined";				
		this.resizableChart = new xsa.monitor.widgets.resizing.ResizingChartWithTableData({
			chartType: this.chartType, 
			chartArray: this.chartArray, 
			yaxis: this.yaxis, 
			yAxisLabel: this.yAxisLabel, 
			xAxisLabel: this.xAxisLabel, 
			chartValueLabel: this.chartValueLabel,
			fill: this.fill,
			url: this.url,
			aggregationScope: "datagrid",
			metrics: this.metrics,
			chartName: this.chartName,
			offset: offset
		});
		dojo.place(this.resizableChart.domNode, this.largeChartContentPane, "last");
		this.resizeLargeChart();
		
		this.resizableChart.startup();
		this.resizableChart.showStandby();
	
		setTimeout(dojo.hitch(this, function(){ // this timeout is needed because we have to wait for the standby to show up before creating the chart.  The
							// sync xhr get stops the standby from fading in otherwise
			this.resizableChart.createChart();
			this.resizeLargeChart();
			this.resizableChart.resizeTabContainer();
			this.resizableChart.hideStandby();
		}), 501);
		
		//a11y
        dojo.forEach(dojo.query("div[role='group']"), function(div){
            if(!dojo.attr(div, "aria-label") || dojo.attr(div, "aria-label")==""){
                dojo.removeAttr(div, "role");
            }
        });

	},
	
	openLargeChart: function(){
		if(!this.resizableChart) {
			this.createLargeChart();
		} else {
			dojo.style(this.largeChartContentPane, "display", "block");
			this.resizeLargeChart();
			this.resizableChart.resizeTabContainer(); //The tab container doesn't resize appropriately until it's 
								  //been visually displayed on the screen. So lets for a resize after we show it.
		}
		this.hideMainContent();		
		this.openLargeChartTooltip.close();
	},
	
	openQueryConsole: function(){
		document.body.style.cursor = "progress";
		if(!this.queryConsoleCP) {
			this.createQueryConsole();
			dojo.place(dojo.create("div", {innerHTML:this.datamgmt_messages.cacheContents,
						    'class': "singleGridLabelWhite left"
			}), this.queryConsoleContentPane, "first");
		}
		this.hideMainContent();
		dojo.style(this.queryConsoleContentPane, "display", "block");
		document.body.style.cursor = "";
		this.openQueryConsoleTooltip.close();
	},
	
	hideMainContent: function(){
		dojo.style(dojo.byId("layoutDiv"), "display", "none");
		dojo.attr(dojo.byId("layoutDiv"), "aria-hidden", "true");
	},
	
	showMainContent: function(){
		dojo.style(dojo.byId("layoutDiv"), "display", "block");
		dojo.attr(dojo.byId("layoutDiv"), "aria-hidden", "false");
	},
	
	uninitialize: function() {
		// Unsubscribe from all the subscriptions...
		dojo.forEach(this.subscriptions, function(item) {
			dojo.unsubscribe(item);
		});
	},

	goBack: function() {
		if(this.queryConsoleContentPane) dojo.style(this.queryConsoleContentPane, "display", "none");
		dojo.style(this.largeChartContentPane, "display", "none");
		this.showMainContent();
	}, 
		
	destroy: function(){
		if(this.largeChartContentPane) dojo.destroy(this.largeChartContentPane);
		if(this.queryConsoleContentPane) queryConsoleContentPane.destroyRecursive();
		if(this.resizableChart) this.resizableChart.destroyRecursive();
		if(this.queryConsole) this.queryConsole.destroyRecursive();
		if(this.openLargeChartTooltip) this.openLargeChartTooltip.destroyRecursive();
		if(this.openQueryConsoleTooltip) this.openQueryConsoleTooltip.destroyRecursive();
		if(this.connection) dojo.disconnect(this.connection);
		dojo.forEach(this.subscriptions, dojo.unsubscribe);
		if(this.buttonConnection) dojo.disconnect(this.buttonConnection);
		if(this.buttonConnection2) dojo.disconnect(this.buttonConnection2);
		
		this.inherited(arguments);
	}
});
