/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.resizing.ResizingChartWithTableData");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("javascript.widgets.GenericDialog");
dojo.require("dijit.layout.TabContainer");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("dojox.grid.DataGrid");
dojo.require("dojo.window");
dojo.require("dojo.date.locale");

dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");
dojo.requireLocalization("xsa.datamgmt.widgets", "xsa_messages");

dojo.declare("xsa.monitor.widgets.resizing.ResizingChartWithTableData", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/resizing/templates/ResizingChartWithTableDataTemplate.html"),
	widgetsInTemplate: true,
	xsa_messages: null,
	datamgmt_messages: null,
	
	subscriptions:null,	
	gridName:null,
	resizingChart:null,
	resizingTable:null,
	url:null,
	aggregationScope:null,
	metrics:null,
	yAxisLabel:"",
	xAxisLabel:"",
	chartValueLabel:"",
	chartWithHourView:"",
		
	chartType:"line",
	standby:null,
	locale:"en",
	
	constructor: function(args){
		this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
		this.datamgmt_messages = dojo.i18n.getLocalization("xsa.datamgmt.widgets", "xsa_messages");
		this.subscriptions = [];
		this.chartTitle = this.xsa_messages.XSA30120;
		this.tableTitle = this.xsa_messages.XSA30121;
		
		this.gridName = args.gridName;
		this.chartArray = args.chartArray;
		this.url = args.url;
		this.aggregationScope = args.aggregationScope;
		this.metrics = args.metrics;
		this.chartName = args.chartName;
		if(args.chartType) this.chartType = args.chartType;
		if(args.yaxis) this.yaxis = args.yaxis;
		if(args.yAxisLabel) this.yAxisLabel = args.yAxisLabel;
		if(args.xAxisLabel) this.xAxisLabel = args.xAxisLabel;
		if(args.chartValueLabel) this.chartValueLabel = args.chartValueLabel;
		if(args.fill) this.fill = args.fill;
		
		this.chartWithHourView = this.xsa_messages.chartWithHourView;

	}, 
	
	postCreate: function(){
		this.standby = new dojox.widget.Standby({target: this.domNode.id, duration:0});
		document.body.appendChild(this.standby.domNode);
		
		try{
			if(djConfig && djConfig.locale) this.locale = djConfig.locale;
		} catch (e) {
			console.log("createTable date is defaulting to english locale");
		}
		
		this.inherited(arguments);
	}, 
	
	startup: function(){
		this.standby.startup();
	
		tabs = this.tabDiv;
		tabs.startup();

		dojo.attr(tabs.domNode, "aria-label", this.chartName);
		this.fixTabForA11ySupport(); //fix it now
		this.tabConnectHandle = dojo.connect(tabs, "onClick", this, "fixTabForA11ySupport"); //fix it again when the tab is clicked
		
		dojo.forEach(dojo.query("div[lang='']"), function(node){
			dojo.removeAttr(node, "lang");
		});
		
		dojo.forEach(dojo.query("div[dir='']"), function(node){
			dojo.removeAttr(node, "dir");
		});
		
		dojo.removeAttr(this.largeChartContentPane.domNode, "role");
		dojo.removeAttr(this.tableContentPane.domNode, "role");
		
		this.subscriptions[this.subscriptions.length] = dojo.subscribe(this.id+"_chart"+"_chartArrayUpdated", dojo.hitch(this, function(message){
			this.updateTable(message.chartArray, message.yAxisLabel);
		}));
		
		this.inherited(arguments);
	},
	
	showStandby: function(){
		this.standby.show(); 
		dojo.style(this.standby._underlayNode, "opacity", 0.75)
		dojo.style(this.standby._imageNode, "opacity", 0.75)
	}, 
	
	hideStandby: function(){
		this.standby.hide();
	}, 
	
	createChart: function(){
		this.resizingChart =  new xsa.monitor.widgets.resizing.ResizingChart({
			gridName: this.gridName,
			id:this.id+"_chart",
			chartArray:this.chartArray,
			url:this.url,
			aggregationScope:this.aggregationScope,
			metrics:this.metrics,
			yAxisLabel:this.yAxisLabel,
			xAxisLabel:this.xAxisLabel,
			chartValueLabel:this.chartValueLabel,
			offset: this.offset
		}, this.chartDiv);
		
		if(this.yaxis) {
			this.resizingChart.setYaxis(this.yaxis);
		}
		if(this.fill) {
			this.resizingChart.fill = this.fill;
		}
		dojo.style(this.largeChartContentPane.domNode, "overflow", "visible");
	},
	
	updateTable: function(chartArray, label){
		if(this.resizingTable) {
			var store = this.createGridStore(chartArray, label);
			if(dojo.hasClass(this.tableContentPane.containerNode, "dijitHidden")) {
				//update table
				this.resizingTable.setStore(store);
			} else {
				//do nothing. to avoid the user losing context, do not update the table if it's visible
			}
		} else {
			this.createTable(chartArray, label);
		}
	},
	
	createTable: function(chartArray, label){
		var store = this.createGridStore(chartArray, label);

		var layout = [[
		  {'name': this.xsa_messages.time, 'field': 'time', 'width': '30%'},
		  {'name': this.datamgmt_messages.value, 'field': 'value', 'width': 'auto'}
		]];

		this.resizingTable = new dojox.grid.DataGrid({
			store: store,
			structure: layout
		}, this.tableDiv);
		
		this.resizingTable.startup();
		
		dojo.attr(this.resizingTable.domNode, "aria-label", this.chartName+" "+this.tableTitle);
	},
	
	getStoreData: function(chartArray, label){
		var data = {
		  identifier: "id",
		  items: []
		};

		for(var i = 0; i < chartArray.length; i++){
			var newDate = new Date( );
			newDate.setTime( chartArray[i][0] );
			var convertedTime = dojo.date.locale.format(newDate, {formatLength: "short", locale: this.locale});
			var value = dojo.number.format(+chartArray[i][1], { locale: this.locale}) + " " + label;
			data.items.push({id:+chartArray[i][0], time:convertedTime, value:value});
		}
		return data;
	},
	
	createGridStore: function(chartArray, label){		
		return new dojo.data.ItemFileWriteStore({data: this.getStoreData(chartArray, label)});
	},
	
	resize: function(height, width){
		dojo.style(this.tabDiv.domNode, "height", height+"px");
		dojo.style(this.tabDiv.domNode, "width", width+"px");
		if(this.resizingChart) this.resizingChart.resize(height*0.85, width*0.98);
	},
	
	resizeTabContainer: function(){
		this.tabDiv.resize();
	},
	
	fixTabForA11ySupport: function() {
		dojo.forEach(dojo.query("div[aria-pressed]"), function(node){
			if(dojo.attr(node, "role") == "tab") {
				dojo.removeAttr(node, "aria-pressed");
			}
		});
	},
	
	uninitialize: function() {
        // Unsubscribe from all the subscriptions...
        dojo.forEach(this.subscriptions, function(item) {
            dojo.unsubscribe(item);
        });
    },
	
	destroy: function(){
		this.resizingChart.destroyRecursive();
		this.resizingTable.destroyRecursive();
		this.tabDiv.destroyRecursive();
		if(this.standby) this.standby.destroyRecursive();
		if(this.tabConnectHandle) dojo.disconnect(this.tabConnectHandle);
		
		this.inherited(arguments);
	}
});
