//Licensed Materials - Property of IBM
//IBM WebSphere DataPower XC10 Appliance (7199-92X).
//Copyright IBM Corporation 2013. All Rights Reserved.
//US Government Users Restricted Rights - Use, duplication or disclosure
//restricted by GSA ADP Schedule Contract with IBM Corp.

dojo.provide("xsa.healthcenter.HealthCenter");

dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.timing");

dojo.requireLocalization("xsa.healthcenter", "xsa_messages");

dojo.declare("xsa.healthcenter.HealthCenter", [dijit._Widget, dijit._Templated], {

	xsa_messages : null,
	
    // templateString: String
    //The string to use for the template of this widget.
    templateString: dojo.cache("xsa", "healthcenter/templates/HealthCenterTemplate.html"),
    
	// pageDescription: [public] String
	//		NLS message to display as the description of the panel.
    pageDescription: "",
    
    state: "",
    _timer: null,

    constructor: function(kwArgs) {
        //this.link = kwArgs.link;
    	this.xsa_messages = dojo.i18n.getLocalization("xsa.healthcenter", "xsa_messages");
    	this.pageDescription = this.xsa_messages.healthCenter;
    },
    
	postCreate: function() {	
		this.inherited(arguments);
		this.getHealth();
		//this.createTimer();
        //this._timer.start();
    },
    
    createTimer: function() {
        this._timer = new dojox.timing.Timer(15000);
        this._timer.onTick = dojo.hitch(this, this._onTick);
        this._timer.onStart = dojo.hitch(this, this._onTick);
    },

    getHealth: function(){
        var xhrArgs = {
              url: '/resources/health_status/',
              content: {},
              sync: true,
              handleAs: "json",
              headers: {"Content-Type" : "text/json"},
              load: dojo.hitch(this, this._refreshData),
              error: function (error) {
                //TODO: handle error
                console.debug("Error", error);
              }
        };
        dojo.xhrGet(xhrArgs);
    },
    
    _refreshData: function(response) {
    	
    	this.prettyPrint(response, 0);
    },

    prettyPrint: function(response, level) {
    	var responseKeys = Object.keys(response);
    	
    	var skipMessageRelatedKeys = false;
    	if (response.STATE != undefined || response.MESSAGE != undefined || response.NAME != undefined) {
    		skipMessageRelatedKeys = true;
    		this.addMessage(level, response);
    	}
    	
    	for (var i=0; i<responseKeys.length; i++) {
    		var itemKey = responseKeys[i];
    		
    		// Skip state, message and name keys b/c we collapse them into a single pretty message.
        	if (skipMessageRelatedKeys && (itemKey == "STATE" || itemKey == "MESSAGE" || itemKey == "NAME")) {
        		continue;
        	}
    		
    		var item = response[itemKey];
    		
    		var printLine = "";
    		for (var j=0; j<level; j++) {
    			printLine += "--"
    		}

    		this.addRow(level, itemKey, item);
    		
    		 if (item != undefined && typeof item == "object") {
	    		var itemKeys = Object.keys(item); 
	    		if (itemKeys != undefined)
	    			this.prettyPrint(item, level+1);
    		} else if (typeof item == "string") {
    			// do nothing.
    		}
    	}
    },
    
    addMessage: function(level, response) {

        var statusSpan = document.createElement('span');
        dojo.attr(statusSpan, { 
        	'id' : 'span-'+response.NAME+'-status',
        	'style' : 'padding-left: '+level*2+'em;',
        	'class' : 'default' 
        });

        if (response.STATE != undefined) {
	        var src = "";
	        var description = "";
	        if (response.STATE == 0) {
	        	src = '/images/success-icon.gif';
	        	description = 'normal';
	        } else if (response.STATE == 1) {
	        	src = '/images/warning-icon.png';
	        	description = 'warning';
	        } else if (response.STATE == 2) {
	        	src = '/images/error-icon.gif';
	        	description = 'error';
	        }
	        
	        var img = document.createElement('img');
	        dojo.attr(img, { 
	        	'src' : src,
	        	'class' : 'icon-image',
	        	'style' : 'vertical-align: middle;',
	        	'alt' : this.xsa_messages[description],
	        	'title' : this.xsa_messages[description]
	        });
	        
	        statusSpan.appendChild(img);
        }
        
        if (response.NAME != undefined) {
	        if (typeof response.NAME == "string")
	        	statusSpan.appendChild(document.createTextNode(" "+response.NAME+" "));
	        else
	        	statusSpan.appendChild(document.createTextNode(" ERROR_NO_NAME "));
        }

        if (response.MESSAGE != undefined) {
	        if (typeof response.MESSAGE == "string")
	        	statusSpan.appendChild(document.createTextNode(" "+response.MESSAGE));
	        else
	        	statusSpan.appendChild(document.createTextNode(" ERROR_NO_MESSAGE"));
        }
        statusSpan.appendChild(document.createElement('br'));
        dojo.place(statusSpan, this._rawDataDiv);
    },
    
    addRow: function(level, itemKey, item) {
        var statusSpan = document.createElement('span');
        dojo.attr(statusSpan, { 
        	'id' : 'span-'+itemKey+'-status',
        	'style' : 'padding-left: '+level*2+'em;',
        	'class' : 'default' 
        });
    	
    	if (itemKey != "TIMESTAMP") {
    		if (item == undefined || typeof item == "string")
	        	statusSpan.appendChild(document.createTextNode(itemKey+" : "+item));
	        else
	        	statusSpan.appendChild(document.createTextNode(itemKey+" : "));
    	} else {
            if (typeof item != "object")
            	statusSpan.appendChild(document.createTextNode(itemKey+" : "+this.convertTime(item)));
            else
            	statusSpan.appendChild(document.createTextNode(itemKey+" : "));
    	}
    	
        statusSpan.appendChild(document.createElement('br'));
        
        // Only place entry if the key is not a number.  If it is a number
        // then we are just looking at the index for a list.
        if (!/^\+?(0|[1-9]\d*)$/.test(itemKey))
        	dojo.place(statusSpan, this._rawDataDiv);
    },
    
    convertTime: function(time) {
		var newDate = new Date( );
		newDate.setTime( time );
		time = dojo.date.locale.format(newDate, {formatLength: "short", locale: djConfig.locale});
        return time;
    },

    destroy: function() {
    	this.inherited(arguments);
    }
});
